@@ -182,7 +182,7 @@ class NestBase(object):
         self._serial = serial
         self._nest_api = nest_api
 
-    def __repr__(self):
+    def __str__(self):
         return '<%s: %s>' % (self.__class__.__name__, self._repr_name)
 
     def _set(self, what, data):
@@ -228,6 +228,9 @@ class Device(NestBase):
 
         return self.where
 
+    def __repr__(self):
+        return str(self._device)
+
     @property
     def name(self):
         return self._device.get('name')
@@ -893,15 +896,15 @@ class SmokeCoAlarm(Device):
 
     @property
     def smoke_sequence_number(self):
-        return self._device['smoke_sequence_number']
+        return self._device.get('smoke_sequence_number')
 
     @property
     def smoke_status(self):
-        return self._device['smoke_alarm_state']
+        return self._device.get('smoke_alarm_state')
 
     @property
     def software_version(self):
-        return self._device['software_version']
+        return self._device.get('software_version')
 
     @property
     def spoken_where_id(self):
@@ -963,7 +966,7 @@ class ActivityZone(NestBase):
     @property
     def _activity_zone(self):
         return next(
-            z for z in self._camera['activity_zones']
+            z for z in self._camera.get('activity_zones')
             if z['id'] == self.zone_id)
 
     @property
@@ -972,7 +975,7 @@ class ActivityZone(NestBase):
 
     @property
     def name(self):
-        return self._activity_zone['name']
+        return self._activity_zone.get('name')
 
 
 class CameraEvent(NestBase):
@@ -988,9 +991,12 @@ class CameraEvent(NestBase):
     def _event(self):
         return self._camera.get('last_event')
 
-    def __repr__(self):
+    def __str__(self):
         return '<%s>' % (self.__class__.__name__)
 
+    def __repr__(self):
+        return str(self._event)
+
     def activity_in_zone(self, zone_id):
         if 'activity_zone_ids' in self._event:
             return str(zone_id) in self._event['activity_zone_ids']
@@ -1185,7 +1191,7 @@ class Camera(Device):
 
     @property
     def model(self):
-        return self._device['model']
+        return self._device.get('model')
 
     @property
     def nexus_api_http_server_url(self):
@@ -1294,7 +1300,7 @@ class Camera(Device):
 
     @property
     def where_id(self):
-        return self._device['where_id']
+        return self._device.get('where_id')
 
     @property
     def wifi_ip_address(self):
@@ -1319,18 +1325,25 @@ class Camera(Device):
         else:
             return SIMULATOR_SNAPSHOT_PLACEHOLDER_URL
 
+    @property
+    def web_url(self):
+        return self._device.get('web_url')
+
 
 class Structure(NestBase):
     @property
     def _structure(self):
         return self._nest_api._status[STRUCTURES][self._serial]
 
+    def __repr__(self):
+        return str(self._structure)
+
     def _set_away(self, value, auto_away=False):
         self._set('structures', {'away': AWAY_MAP[value]})
 
     @property
     def away(self):
-        return self._structure['away']
+        return self._structure.get('away')
 
     @away.setter
     def away(self, value):
@@ -1425,7 +1438,7 @@ class Structure(NestBase):
 
     @name.setter
     def name(self, value):
-        self._set('structure', {'name': value})
+        self._set('structures', {'name': value})
 
     @property
     def location(self):
@@ -1482,8 +1495,7 @@ class Structure(NestBase):
 
     @property
     def time_zone(self):
-        if 'time_zone' in self._structure:
-            return self._structure['time_zone']
+        return self._structure.get('time_zone')
 
     @property
     def peak_period_start_time(self):
@@ -1500,9 +1512,32 @@ class Structure(NestBase):
         if 'eta_begin' in self._structure:
             return parse_time(self._structure['eta_begin'])
 
+    def set_eta(self, trip_id, eta_begin, eta_end=None):
+        """
+        Set estimated arrival winow, use same trip_id to update estimation.
+        Nest may choose ignore inaccurate estimation.
+        See: https://developers.nest.com/documentation/cloud/away-guide
+             #make_an_eta_write_call
+        """
+        if self.num_thermostats == 0:
+            raise ValueError("ETA can only be set when Thermostat is in the"
+                             " structure.")
+        if trip_id is None:
+            raise ValueError("trip_id must be not None")
+        if eta_begin is None:
+            raise ValueError("eta_begin must be not None")
+        if eta_end is None:
+            # set default eta window to 1 minute
+            eta_end = eta_begin + datetime.timedelta(0, 60)
+
+        data = {'trip_id': trip_id,
+                'estimated_arrival_window_begin': eta_begin.isoformat(),
+                'estimated_arrival_window_end': eta_end.isoformat()}
+        self._set('structures', {'eta': data})
+
     @property
     def wheres(self):
-        return self._structure['wheres']
+        return self._structure.get('wheres')
 
     @wheres.setter
     def wheres(self, value):
@@ -1539,6 +1574,15 @@ class Structure(NestBase):
         self.wheres = wheres
         return ident
 
+    @property
+    def security_state(self):
+        """
+        Return 'ok' or 'deter'. Need sercurity state ready permission.
+        Note: this is NOT for Net Secruity alarm system.
+        See https://developers.nest.com/documentation/cloud/security-guide
+        """
+        return self._structure.get('wwn_security_state')
+
 
 class Nest(object):
     def __init__(self, username=None, password=None,
@@ -1781,9 +1825,7 @@ class Nest(object):
             self._open_data_stream("/")
         self._queue_lock.release()
 
-        self._queue_lock.acquire(False)
         value = self._queue[0]['data']
-        self._queue_lock.release()
         if not value:
             value = self._get("/")
 
